@@ -18,11 +18,12 @@
  */
 package org.apache.samza.container.grouper.stream;
 
-import java.util.*;
-
 import org.apache.samza.container.TaskName;
 import org.apache.samza.system.SystemStreamPartition;
 
+import java.util.Map;
+import java.util.Set;
+
 /**
  * Group a set of SystemStreamPartitions into logical taskNames that share a common characteristic, defined
  * by the implementation.  Each taskName has a key that uniquely describes what sets may be in it, but does
