@@ -19,8 +19,8 @@
 
 package org.apache.samza.system;
 
-import org.codehaus.jackson.annotate.JsonCreator;
-import org.codehaus.jackson.annotate.JsonProperty;
+import com.fasterxml.jackson.annotation.JsonCreator;
+import com.fasterxml.jackson.annotation.JsonProperty;
 
 /**
  *  The EndOfStreamMessage is a control message that is sent out to next stage
