@@ -55,7 +55,7 @@ import org.apache.samza.system.eventhub.EventHubClientManagerFactory;
 import org.apache.samza.system.eventhub.EventHubConfig;
 import org.apache.samza.system.eventhub.Interceptor;
 import org.apache.samza.system.eventhub.admin.EventHubSystemAdmin;
-import org.apache.samza.system.eventhub.metrics.SamzaHistogram;
+import org.apache.samza.metrics.SamzaHistogram;
 import org.apache.samza.system.eventhub.producer.EventHubSystemProducer;
 import org.apache.samza.util.BlockingEnvelopeMap;
 import org.apache.samza.util.Clock;
@@ -104,7 +104,9 @@ public class EventHubSystemConsumer extends BlockingEnvelopeMap {
   private static final Logger LOG = LoggerFactory.getLogger(EventHubSystemConsumer.class);
 
   // Overall timeout for EventHubClient exponential backoff policy
-  private static final Duration DEFAULT_EVENTHUB_RECEIVER_TIMEOUT = Duration.ofMinutes(10L);
+  private static final Duration DEFAULT_EVENTHUB_RECEIVER_TIMEOUT = Duration.ofMinutes(10);
+  private static final Duration DEFAULT_EVENTHUB_CREATE_RECEIVER_TIMEOUT = Duration.ofMinutes(1);
+
   private static final long DEFAULT_SHUTDOWN_TIMEOUT_MILLIS = Duration.ofSeconds(15).toMillis();
 
   public static final String START_OF_STREAM = ClientConstants.START_OF_STREAM; // -1
@@ -274,13 +276,13 @@ public class EventHubSystemConsumer extends BlockingEnvelopeMap {
           // If the offset is greater than the newest offset, use the use current Instant as
           // offset to fetch in Eventhub.
           receiver = eventHubClientManager.getEventHubClient()
-              .createReceiverSync(consumerGroup, partitionId.toString(), EventPosition.fromEnqueuedTime(Instant.now()));
+              .createReceiver(consumerGroup, partitionId.toString(), EventPosition.fromEnqueuedTime(Instant.now())).get(DEFAULT_EVENTHUB_CREATE_RECEIVER_TIMEOUT.toMillis(), TimeUnit.MILLISECONDS);
         } else {
           // EventHub will return the first message AFTER the offset that was specified in the fetch request.
           // If no such offset exists Eventhub will return an error.
           receiver = eventHubClientManager.getEventHubClient()
-              .createReceiverSync(consumerGroup, partitionId.toString(),
-                  EventPosition.fromOffset(offset, /* inclusiveFlag */false));
+              .createReceiver(consumerGroup, partitionId.toString(),
+                  EventPosition.fromOffset(offset, /* inclusiveFlag */false)).get(DEFAULT_EVENTHUB_CREATE_RECEIVER_TIMEOUT.toMillis(), TimeUnit.MILLISECONDS);
         }
 
         receiver.setPrefetchCount(prefetchCount);
@@ -394,28 +396,24 @@ public class EventHubSystemConsumer extends BlockingEnvelopeMap {
   private synchronized void shutdownEventHubsManagers() {
     // There could be potentially many Receivers and EventHubManagers, so close the managers in parallel
     LOG.info("Start shutting down eventhubs receivers");
-    ShutdownUtil.boundedShutdown(streamPartitionReceivers.values().stream().map(receiver -> new Runnable() {
-      @Override
-      public void run() {
+    ShutdownUtil.boundedShutdown(streamPartitionReceivers.values().stream().map(receiver ->
+      (Runnable) () -> {
         try {
           receiver.close().get(DEFAULT_SHUTDOWN_TIMEOUT_MILLIS, TimeUnit.MILLISECONDS);
         } catch (Exception e) {
           LOG.error("Failed to shutdown receiver.", e);
         }
-      }
-    }).collect(Collectors.toList()), "EventHubSystemConsumer.Receiver#close", DEFAULT_SHUTDOWN_TIMEOUT_MILLIS);
+      }).collect(Collectors.toList()), "EventHubSystemConsumer.Receiver#close", DEFAULT_SHUTDOWN_TIMEOUT_MILLIS);
 
     LOG.info("Start shutting down eventhubs managers");
-    ShutdownUtil.boundedShutdown(perPartitionEventHubManagers.values().stream().map(manager -> new Runnable() {
-      @Override
-      public void run() {
+    ShutdownUtil.boundedShutdown(perPartitionEventHubManagers.values().stream().map(manager ->
+      (Runnable) () -> {
         try {
           manager.close(DEFAULT_SHUTDOWN_TIMEOUT_MILLIS);
         } catch (Exception e) {
           LOG.error("Failed to shutdown eventhubs manager.", e);
         }
-      }
-    }).collect(Collectors.toList()), "EventHubSystemConsumer.ClientManager#close", DEFAULT_SHUTDOWN_TIMEOUT_MILLIS);
+      }).collect(Collectors.toList()), "EventHubSystemConsumer.ClientManager#close", DEFAULT_SHUTDOWN_TIMEOUT_MILLIS);
 
     perPartitionEventHubManagers.clear();
     perStreamEventHubManagers.clear();
@@ -447,7 +445,7 @@ public class EventHubSystemConsumer extends BlockingEnvelopeMap {
     private final Counter errorRate;
     private final Interceptor interceptor;
     private final Integer maxEventCount;
-    SystemStreamPartition ssp;
+    private final SystemStreamPartition ssp;
 
     PartitionReceiverHandlerImpl(SystemStreamPartition ssp, Counter eventReadRate, Counter eventByteReadRate,
         SamzaHistogram readLatency, Counter readErrors, Interceptor interceptor, int maxEventCount) {
@@ -469,29 +467,29 @@ public class EventHubSystemConsumer extends BlockingEnvelopeMap {
     public void onReceive(Iterable<EventData> events) {
       if (events != null) {
         events.forEach(event -> {
-            byte[] eventDataBody = event.getBytes();
-            if (interceptor != null) {
-              eventDataBody = interceptor.intercept(eventDataBody);
-            }
-            String offset = event.getSystemProperties().getOffset();
-            Object partitionKey = event.getSystemProperties().getPartitionKey();
-            if (partitionKey == null) {
-              partitionKey = event.getProperties().get(EventHubSystemProducer.KEY);
-            }
-            try {
-              updateMetrics(event);
+          byte[] eventDataBody = event.getBytes();
+          if (interceptor != null) {
+            eventDataBody = interceptor.intercept(eventDataBody);
+          }
+          String offset = event.getSystemProperties().getOffset();
+          Object partitionKey = event.getSystemProperties().getPartitionKey();
+          if (partitionKey == null) {
+            partitionKey = event.getProperties().get(EventHubSystemProducer.KEY);
+          }
+          try {
+            updateMetrics(event);
 
-              // note that the partition key can be null
-              put(ssp, new EventHubIncomingMessageEnvelope(ssp, offset, partitionKey, eventDataBody, event));
-            } catch (InterruptedException e) {
-              String msg = String.format("Interrupted while adding the event from ssp %s to dispatch queue.", ssp);
-              LOG.error(msg, e);
-              throw new SamzaException(msg, e);
-            }
+            // note that the partition key can be null
+            put(ssp, new EventHubIncomingMessageEnvelope(ssp, offset, partitionKey, eventDataBody, event));
+          } catch (InterruptedException e) {
+            String msg = String.format("Interrupted while adding the event from ssp %s to dispatch queue.", ssp);
+            LOG.error(msg, e);
+            throw new SamzaException(msg, e);
+          }
 
-            // Cache latest checkpoint
-            streamPartitionOffsets.put(ssp, offset);
-          });
+          // Cache latest checkpoint
+          streamPartitionOffsets.put(ssp, offset);
+        });
       }
     }
 
@@ -521,7 +519,7 @@ public class EventHubSystemConsumer extends BlockingEnvelopeMap {
               throwable);
           try {
             // Add a fixed delay so that we don't keep retrying when there are long-lasting failures
-            Thread.sleep(Duration.ofSeconds(2).toMillis());
+            TimeUnit.SECONDS.sleep(2);
           } catch (InterruptedException e) {
             LOG.warn("Interrupted during sleep before renew", e);
           }
