@@ -63,6 +63,18 @@ public class JobConfig extends MapConfig {
   public static final String JOB_CONTAINER_COUNT = "job.container.count";
   static final int DEFAULT_JOB_CONTAINER_COUNT = 1;
   public static final String JOB_CONTAINER_THREAD_POOL_SIZE = "job.container.thread.pool.size";
+  // num commit threads == min(max(2 * num tasks in container, thread pool size), max thread pool size)
+  public static final String COMMIT_THREAD_POOL_SIZE = "job.container.commit.thread.pool.size";
+  static final int DEFAULT_COMMIT_THREAD_POOL_SIZE = 2;
+  public static final String COMMIT_THREAD_POOL_MAX_SIZE = "job.container.commit.thread.pool.max.size";
+  static final int DEFAULT_COMMIT_THREAD_POOL_MAX_SIZE = 64;
+
+  // num commit threads == min(max(2 * num tasks in container, thread pool size), max thread pool size)
+  public static final String RESTORE_THREAD_POOL_SIZE = "job.container.restore.thread.pool.size";
+  static final int DEFAULT_RESTORE_THREAD_POOL_SIZE = 2;
+  public static final String RESTORE_THREAD_POOL_MAX_SIZE = "job.container.restore.thread.pool.max.size";
+  static final int DEFAULT_RESTORE_THREAD_POOL_MAX_SIZE = 64;
+
   public static final String JOB_INTERMEDIATE_STREAM_PARTITIONS = "job.intermediate.stream.partitions";
 
   public static final String JOB_DEBOUNCE_TIME_MS = "job.debounce.time.ms";
@@ -143,8 +155,6 @@ public class JobConfig extends MapConfig {
   public static final String COORDINATOR_STREAM_FACTORY = "job.coordinatorstream.config.factory";
   public static final String DEFAULT_COORDINATOR_STREAM_CONFIG_FACTORY = "org.apache.samza.util.DefaultCoordinatorStreamConfigFactory";
 
-  public static final String JOB_SPLIT_DEPLOYMENT_ENABLED = "job.split.deployment.enabled";
-
   private static final String JOB_STARTPOINT_ENABLED = "job.startpoint.enabled";
 
   // Enable ClusterBasedJobCoordinator aka ApplicationMaster High Availability (AM-HA).
@@ -162,6 +172,15 @@ public class JobConfig extends MapConfig {
   public static final String YARN_CONTAINER_HEARTBEAT_RETRY_SLEEP_DURATION_MS = "yarn.container.heartbeat.retry-sleep-duration.ms";
   public static final long YARN_CONTAINER_HEARTBEAT_RETRY_SLEEP_DURATION_MS_DEFAULT = 10000;
 
+  public static final String CONTAINER_HEARTBEAT_MONITOR_ENABLED = "job.container.heartbeat.monitor.enabled";
+  private static final boolean CONTAINER_HEARTBEAT_MONITOR_ENABLED_DEFAULT = true;
+
+
+  // Enabled elasticity for the job
+  // number of (elastic) tasks in the job will be old task count X elasticity factor
+  public static final String JOB_ELASTICITY_FACTOR = "job.elasticity.factor";
+  public static final int DEFAULT_JOB_ELASTICITY_FACTOR = 1;
+
   public JobConfig(Config config) {
     super(config);
   }
@@ -336,6 +355,22 @@ public class JobConfig extends MapConfig {
     }
   }
 
+  public int getCommitThreadPoolSize() {
+    return getInt(COMMIT_THREAD_POOL_SIZE, DEFAULT_COMMIT_THREAD_POOL_SIZE);
+  }
+
+  public int getCommitThreadPoolMaxSize() {
+    return getInt(COMMIT_THREAD_POOL_MAX_SIZE, DEFAULT_COMMIT_THREAD_POOL_MAX_SIZE);
+  }
+
+  public int getRestoreThreadPoolSize() {
+    return getInt(RESTORE_THREAD_POOL_SIZE, DEFAULT_RESTORE_THREAD_POOL_SIZE);
+  }
+
+  public int getRestoreThreadPoolMaxSize() {
+    return getInt(RESTORE_THREAD_POOL_MAX_SIZE, DEFAULT_RESTORE_THREAD_POOL_MAX_SIZE);
+  }
+
   public int getDebounceTimeMs() {
     return getInt(JOB_DEBOUNCE_TIME_MS, DEFAULT_DEBOUNCE_TIME_MS);
   }
@@ -386,10 +421,6 @@ public class JobConfig extends MapConfig {
     return getStandbyTaskReplicationFactor() > 1;
   }
 
-  public boolean isSplitDeploymentEnabled() {
-    return getBoolean(JOB_SPLIT_DEPLOYMENT_ENABLED, false);
-  }
-
   /**
    * The metadata file is written in a {@code exec-env-container-id}.metadata file in the log-dir of the container.
    * Here the {@code exec-env-container-id} refers to the ID assigned by the cluster manager (e.g., YARN) to the container,
@@ -437,4 +468,20 @@ public class JobConfig extends MapConfig {
   public boolean getStartpointEnabled() {
     return getBoolean(JOB_STARTPOINT_ENABLED, true);
   }
+
+  public boolean getContainerHeartbeatMonitorEnabled() {
+    return getBoolean(CONTAINER_HEARTBEAT_MONITOR_ENABLED, CONTAINER_HEARTBEAT_MONITOR_ENABLED_DEFAULT);
+  }
+
+  public boolean getElasticityEnabled() {
+    return getElasticityFactor() > 1;
+  }
+
+  public int getElasticityFactor() {
+    int elasticityFactor = getInt(JOB_ELASTICITY_FACTOR, DEFAULT_JOB_ELASTICITY_FACTOR);
+    if (elasticityFactor < 1) {
+      throw new ConfigException("Elasticity factor can not be less than 1");
+    }
+    return elasticityFactor;
+  }
 }
\ No newline at end of file
