@@ -16,12 +16,13 @@
  * specific language governing permissions and limitations
  * under the License.
  */
-
 package org.apache.samza.config;
 
+import java.util.Optional;
 import com.google.common.base.Strings;
 import org.apache.samza.SamzaException;
 import org.apache.samza.coordinator.CoordinationUtilsFactory;
+import org.apache.samza.coordinator.lifecycle.NoOpJobRestartSignalFactory;
 import org.apache.samza.standalone.PassthroughCoordinationUtilsFactory;
 import org.apache.samza.standalone.PassthroughJobCoordinatorFactory;
 import org.apache.samza.util.ReflectionUtil;
@@ -31,8 +32,11 @@ import org.apache.samza.zk.ZkJobCoordinatorFactory;
 public class JobCoordinatorConfig extends MapConfig {
   public static final String JOB_COORDINATOR_FACTORY = "job.coordinator.factory";
   public final static String DEFAULT_COORDINATOR_FACTORY = ZkJobCoordinatorFactory.class.getName();
+  public static final String JOB_RESTART_SIGNAL_FACTORY = "job.coordinator.restart.signal.factory";
+
   private static final String AZURE_COORDINATION_UTILS_FACTORY = "org.apache.samza.coordinator.AzureCoordinationUtilsFactory";
   private static final String AZURE_COORDINATOR_FACTORY = "org.apache.samza.coordinator.AzureJobCoordinatorFactory";
+  private static final String DEFAULT_JOB_RESTART_SIGNAL_FACTORY = NoOpJobRestartSignalFactory.class.getName();
 
   public JobCoordinatorConfig(Config config) {
     super(config);
@@ -70,11 +74,16 @@ public class JobCoordinatorConfig extends MapConfig {
   }
 
   public String getJobCoordinatorFactoryClassName() {
-    String jobCoordinatorFactoryClassName = get(JOB_COORDINATOR_FACTORY);
-    if (Strings.isNullOrEmpty(jobCoordinatorFactoryClassName)) {
-      return ZkJobCoordinatorFactory.class.getName();
-    } else {
-      return jobCoordinatorFactoryClassName;
-    }
+    return getOptionalJobCoordinatorFactoryClassName()
+        .filter(className -> !Strings.isNullOrEmpty(className))
+        .orElse(ZkJobCoordinatorFactory.class.getName());
+  }
+
+  public Optional<String> getOptionalJobCoordinatorFactoryClassName() {
+    return Optional.ofNullable(get(JOB_COORDINATOR_FACTORY));
+  }
+
+  public String getJobRestartSignalFactory() {
+    return get(JOB_RESTART_SIGNAL_FACTORY, DEFAULT_JOB_RESTART_SIGNAL_FACTORY);
   }
 }
