@@ -18,6 +18,9 @@
  */
 package org.apache.samza.container.host;
 
+import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.List;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -33,48 +36,49 @@ public class PosixCommandBasedStatisticsGetter implements SystemStatisticsGetter
   private static final Logger log = LoggerFactory.getLogger(PosixCommandBasedStatisticsGetter.class);
 
   /**
-   * A convenience method to execute shell commands and return the first line of their output.
+   * A convenience method to execute shell commands and return all lines of their output.
    *
    * @param cmdArray the command to run
-   * @return the first line of the output.
+   * @return all lines of the output.
    * @throws IOException
    */
-  private String getCommandOutput(String[] cmdArray) throws IOException {
+  private List<String> getAllCommandOutput(String[] cmdArray) throws IOException {
+    log.debug("Executing commands {}", Arrays.toString(cmdArray));
     Process executable = Runtime.getRuntime().exec(cmdArray);
-    BufferedReader processReader = null;
-    String psOutput = null;
+    BufferedReader processReader;
+    List<String> psOutput = new ArrayList<>();
 
-    try {
-      processReader = new BufferedReader(new InputStreamReader(executable.getInputStream()));
-      psOutput = processReader.readLine();
-    } finally {
-      if (processReader != null) {
-        processReader.close();
+    processReader = new BufferedReader(new InputStreamReader(executable.getInputStream()));
+    String line;
+    while ((line = processReader.readLine()) != null) {
+      if (!line.isEmpty()) {
+        psOutput.add(line);
       }
     }
+    processReader.close();
     return psOutput;
   }
 
-  private long getPhysicalMemory() throws IOException {
-
-    // returns a single long value that represents the rss memory of the process.
-    String commandOutput = getCommandOutput(new String[]{"sh", "-c", "ps -o rss= -p $PPID"});
-
-    // this should never happen.
-    if (commandOutput == null) {
-      throw new IOException("ps returned unexpected output: " + commandOutput);
+  private long getTotalPhysicalMemoryUsageBytes() throws IOException {
+    // collect all child process ids of the main process that runs the application
+    List<String> processIds = getAllCommandOutput(new String[]{"sh", "-c", "pgrep -P $PPID"});
+    // add the parent process which is the main process that runs the application
+    processIds.add("$PPID");
+    String processIdsJoined = String.join(" ", processIds);
+    // returns a list of long values that represent the rss memory of each process.
+    List<String> processMemoryKBArray = getAllCommandOutput(new String[]{"sh", "-c", String.format("ps -o rss= -p %s", processIdsJoined)});
+    long totalPhysicalMemoryKB = 0;
+    for (String processMemory : processMemoryKBArray) {
+      totalPhysicalMemoryKB += Long.parseLong(processMemory.trim());
     }
-
-    long rssMemoryKb = Long.parseLong(commandOutput.trim());
     //convert to bytes
-    return rssMemoryKb * 1024;
+    return totalPhysicalMemoryKB * 1024;
   }
 
-
   @Override
   public SystemMemoryStatistics getSystemMemoryStatistics() {
     try {
-      long memory = getPhysicalMemory();
+      long memory = getTotalPhysicalMemoryUsageBytes();
       return new SystemMemoryStatistics(memory);
     } catch (Exception e) {
       log.warn("Error when running ps: ", e);
