@@ -19,6 +19,9 @@
 
 package org.apache.samza.metrics;
 
+import java.util.List;
+
+
 /**
  * MetricGroup is a little helper class to make it easy to register and
  * manage a group of counters, gauges and timers.  It's shared between Java
@@ -44,10 +47,6 @@ public class MetricGroup {
     return registry.newCounter(groupName, (prefix + name).toLowerCase());
   }
 
-  public <T> ListGauge<T> newListGauge(String name) {
-    return registry.newListGauge(groupName, new ListGauge(name));
-  }
-
   public <T> Gauge<T> newGauge(String name, T value) {
     return registry.newGauge(groupName, new Gauge<T>((prefix + name).toLowerCase(), value));
   }
@@ -69,4 +68,14 @@ public class MetricGroup {
   public Timer newTimer(String name) {
     return registry.newTimer(groupName, (prefix + name).toLowerCase());
   }
+
+  /**
+   *
+   * @param name - Name of the histogram metric
+   * @param percentiles - percentiles you want to record for this metric, such as P50, P90, P99
+   * @return Return a SamzaHistogram to record the percentiles of a specific metric.
+   */
+  public SamzaHistogram newHistogram(String name, List<Double> percentiles) {
+    return new SamzaHistogram(registry, groupName, (prefix + name).toLowerCase(), percentiles);
+  }
 }
