@@ -22,12 +22,12 @@ package org.apache.samza.runtime;
 import java.time.Duration;
 import java.util.List;
 import org.apache.samza.SamzaException;
-import org.apache.samza.application.descriptors.ApplicationDescriptor;
-import org.apache.samza.application.descriptors.ApplicationDescriptorImpl;
 import org.apache.samza.application.descriptors.ApplicationDescriptorUtil;
 import org.apache.samza.application.SamzaApplication;
 import org.apache.samza.config.Config;
 import org.apache.samza.config.JobConfig;
+import org.apache.samza.context.ExternalContext;
+import org.apache.samza.execution.JobPlanner;
 import org.apache.samza.execution.RemoteJobPlanner;
 import org.apache.samza.job.ApplicationStatus;
 import org.apache.samza.job.JobRunner;
@@ -45,8 +45,8 @@ public class RemoteApplicationRunner implements ApplicationRunner {
   private static final Logger LOG = LoggerFactory.getLogger(RemoteApplicationRunner.class);
   private static final long DEFAULT_SLEEP_DURATION_MS = 2000;
 
-  private final ApplicationDescriptorImpl<? extends ApplicationDescriptor> appDesc;
-  private final RemoteJobPlanner planner;
+  private final SamzaApplication app;
+  private final Config config;
 
   /**
    * Constructors a {@link RemoteApplicationRunner} to run the {@code app} with the {@code config}.
@@ -55,13 +55,21 @@ public class RemoteApplicationRunner implements ApplicationRunner {
    * @param config configuration for the application
    */
   public RemoteApplicationRunner(SamzaApplication app, Config config) {
-    this.appDesc = ApplicationDescriptorUtil.getAppDescriptor(app, config);
-    this.planner = new RemoteJobPlanner(appDesc);
+    this.app = app;
+    this.config = config;
   }
 
   @Override
-  public void run() {
+  public void run(ExternalContext externalContext) {
+    if (new JobConfig(config).getConfigLoaderFactory().isPresent()) {
+      JobRunner runner = new JobRunner(JobPlanner.generateSingleJobConfig(config));
+      runner.submit();
+      return;
+    }
+
+    // TODO SAMZA-2432: Clean this up once SAMZA-2405 is completed when legacy flow is removed.
     try {
+      JobPlanner planner = new RemoteJobPlanner(ApplicationDescriptorUtil.getAppDescriptor(app, config));
       List<JobConfig> jobConfigs = planner.prepareJobs();
       if (jobConfigs.isEmpty()) {
         throw new SamzaException("No jobs to run.");
@@ -69,10 +77,10 @@ public class RemoteApplicationRunner implements ApplicationRunner {
 
       // 3. submit jobs for remote execution
       jobConfigs.forEach(jobConfig -> {
-          LOG.info("Starting job {} with config {}", jobConfig.getName(), jobConfig);
-          JobRunner runner = new JobRunner(jobConfig);
-          runner.run(true);
-        });
+        LOG.info("Starting job {} with config {}", jobConfig.getName(), jobConfig);
+        JobRunner runner = new JobRunner(jobConfig);
+        runner.run(true);
+      });
     } catch (Throwable t) {
       throw new SamzaException("Failed to run application", t);
     }
@@ -83,7 +91,7 @@ public class RemoteApplicationRunner implements ApplicationRunner {
     // since currently we only support single actual remote job, we can get its status without
     // building the execution plan.
     try {
-      JobConfig jc = new JobConfig(appDesc.getConfig());
+      JobConfig jc = new JobConfig(JobPlanner.generateSingleJobConfig(config));
       LOG.info("Killing job {}", jc.getName());
       JobRunner runner = new JobRunner(jc);
       runner.kill();
@@ -97,7 +105,7 @@ public class RemoteApplicationRunner implements ApplicationRunner {
     // since currently we only support single actual remote job, we can get its status without
     // building the execution plan
     try {
-      JobConfig jc = new JobConfig(appDesc.getConfig());
+      JobConfig jc = new JobConfig(JobPlanner.generateSingleJobConfig(config));
       return getApplicationStatus(jc);
     } catch (Throwable t) {
       throw new SamzaException("Failed to get status for application", t);
@@ -111,7 +119,7 @@ public class RemoteApplicationRunner implements ApplicationRunner {
 
   @Override
   public boolean waitForFinish(Duration timeout) {
-    JobConfig jobConfig = new JobConfig(appDesc.getConfig());
+    JobConfig jobConfig = new JobConfig(JobPlanner.generateSingleJobConfig(config));
     boolean finished = true;
     long timeoutInMs = timeout.toMillis();
     long startTimeInMs = System.currentTimeMillis();
