@@ -19,11 +19,32 @@
 
 package org.apache.samza.serializers.model;
 
-import java.io.IOException;
-import java.util.HashMap;
-import java.util.Map;
+import com.fasterxml.jackson.core.JsonGenerator;
+import com.fasterxml.jackson.core.JsonParser;
+import com.fasterxml.jackson.core.JsonProcessingException;
+import com.fasterxml.jackson.core.ObjectCodec;
+import com.fasterxml.jackson.core.Version;
+import com.fasterxml.jackson.core.type.TypeReference;
+import com.fasterxml.jackson.databind.DeserializationContext;
+import com.fasterxml.jackson.databind.DeserializationFeature;
+import com.fasterxml.jackson.databind.JsonDeserializer;
+import com.fasterxml.jackson.databind.JsonNode;
+import com.fasterxml.jackson.databind.JsonSerializer;
+import com.fasterxml.jackson.databind.KeyDeserializer;
+import com.fasterxml.jackson.databind.ObjectMapper;
+import com.fasterxml.jackson.databind.PropertyNamingStrategy;
+import com.fasterxml.jackson.databind.SerializationFeature;
+import com.fasterxml.jackson.databind.SerializerProvider;
+import com.fasterxml.jackson.databind.cfg.MapperConfig;
+import com.fasterxml.jackson.databind.introspect.AnnotatedField;
+import com.fasterxml.jackson.databind.introspect.AnnotatedMethod;
+import com.fasterxml.jackson.databind.module.SimpleModule;
+import com.fasterxml.jackson.datatype.jdk8.Jdk8Module;
 import org.apache.samza.Partition;
 import org.apache.samza.SamzaException;
+import org.apache.samza.checkpoint.CheckpointId;
+import org.apache.samza.checkpoint.CheckpointV2;
+import org.apache.samza.checkpoint.kafka.KafkaStateCheckpointMarker;
 import org.apache.samza.config.Config;
 import org.apache.samza.config.MapConfig;
 import org.apache.samza.container.TaskName;
@@ -36,25 +57,9 @@ import org.apache.samza.job.model.TaskMode;
 import org.apache.samza.job.model.TaskModel;
 import org.apache.samza.system.SystemStream;
 import org.apache.samza.system.SystemStreamPartition;
-import org.codehaus.jackson.JsonGenerator;
-import org.codehaus.jackson.JsonNode;
-import org.codehaus.jackson.JsonParser;
-import org.codehaus.jackson.JsonProcessingException;
-import org.codehaus.jackson.ObjectCodec;
-import org.codehaus.jackson.Version;
-import org.codehaus.jackson.map.DeserializationContext;
-import org.codehaus.jackson.map.JsonDeserializer;
-import org.codehaus.jackson.map.JsonSerializer;
-import org.codehaus.jackson.map.KeyDeserializer;
-import org.codehaus.jackson.map.MapperConfig;
-import org.codehaus.jackson.map.ObjectMapper;
-import org.codehaus.jackson.map.PropertyNamingStrategy;
-import org.codehaus.jackson.map.SerializerProvider;
-import org.codehaus.jackson.map.introspect.AnnotatedField;
-import org.codehaus.jackson.map.introspect.AnnotatedMethod;
-import org.codehaus.jackson.map.module.SimpleModule;
-import org.codehaus.jackson.type.TypeReference;
-
+import java.io.IOException;
+import java.util.HashMap;
+import java.util.Map;
 /**
  * <p>
  * A collection of utility classes and (de)serializers to make Samza's job model
@@ -78,6 +83,8 @@ public class SamzaObjectMapper {
    */
   public static ObjectMapper getObjectMapper() {
     ObjectMapper mapper = new ObjectMapper();
+    mapper.configure(DeserializationFeature.WRAP_EXCEPTIONS, false);
+    mapper.configure(SerializationFeature.WRAP_EXCEPTIONS, false);
     SimpleModule module = new SimpleModule("SamzaModule", new Version(1, 0, 0, ""));
 
     // Setup custom serdes for simple data types.
@@ -92,13 +99,15 @@ public class SamzaObjectMapper {
     module.addKeyDeserializer(SystemStreamPartition.class, new SystemStreamPartitionKeyDeserializer());
     module.addDeserializer(Config.class, new ConfigDeserializer());
     module.addDeserializer(TaskMode.class, new TaskModeDeserializer());
+    module.addSerializer(CheckpointId.class, new CheckpointIdSerializer());
+    module.addDeserializer(CheckpointId.class, new CheckpointIdDeserializer());
 
     // Setup mixins for data models.
-    mapper.getSerializationConfig().addMixInAnnotations(TaskModel.class, JsonTaskModelMixIn.class);
-    mapper.getDeserializationConfig().addMixInAnnotations(TaskModel.class, JsonTaskModelMixIn.class);
-    mapper.getSerializationConfig().addMixInAnnotations(ContainerModel.class, JsonContainerModelMixIn.class);
-    mapper.getSerializationConfig().addMixInAnnotations(JobModel.class, JsonJobModelMixIn.class);
-    mapper.getDeserializationConfig().addMixInAnnotations(JobModel.class, JsonJobModelMixIn.class);
+    mapper.addMixIn(TaskModel.class, JsonTaskModelMixIn.class);
+    mapper.addMixIn(ContainerModel.class, JsonContainerModelMixIn.class);
+    mapper.addMixIn(JobModel.class, JsonJobModelMixIn.class);
+    mapper.addMixIn(CheckpointV2.class, JsonCheckpointV2Mixin.class);
+    mapper.addMixIn(KafkaStateCheckpointMarker.class, KafkaStateCheckpointMarkerMixin.class);
 
     module.addDeserializer(ContainerModel.class, new JsonDeserializer<ContainerModel>() {
       @Override
@@ -122,31 +131,26 @@ public class SamzaObjectMapper {
                 String.format("JobModel was missing %s and %s. This should never happen. JobModel corrupt!",
                     JsonContainerModelMixIn.PROCESSOR_ID_KEY, JsonContainerModelMixIn.CONTAINER_ID_KEY));
           }
-          id = String.valueOf(node.get(JsonContainerModelMixIn.CONTAINER_ID_KEY).getIntValue());
+          id = String.valueOf(node.get(JsonContainerModelMixIn.CONTAINER_ID_KEY).intValue());
         } else {
-          id = node.get(JsonContainerModelMixIn.PROCESSOR_ID_KEY).getTextValue();
+          id = node.get(JsonContainerModelMixIn.PROCESSOR_ID_KEY).textValue();
         }
         Map<TaskName, TaskModel> tasksMapping =
-            OBJECT_MAPPER.readValue(node.get(JsonContainerModelMixIn.TASKS_KEY),
+            OBJECT_MAPPER.readValue(OBJECT_MAPPER.treeAsTokens(node.get(JsonContainerModelMixIn.TASKS_KEY)),
                 new TypeReference<Map<TaskName, TaskModel>>() { });
         return new ContainerModel(id, tasksMapping);
       }
     });
 
-    mapper.getSerializationConfig().addMixInAnnotations(LocalityModel.class, JsonLocalityModelMixIn.class);
-    mapper.getDeserializationConfig().addMixInAnnotations(LocalityModel.class, JsonLocalityModelMixIn.class);
-    mapper.getSerializationConfig().addMixInAnnotations(ProcessorLocality.class, JsonProcessorLocalityMixIn.class);
-    mapper.getDeserializationConfig().addMixInAnnotations(ProcessorLocality.class, JsonProcessorLocalityMixIn.class);
+    mapper.addMixIn(LocalityModel.class, JsonLocalityModelMixIn.class);
+    mapper.addMixIn(ProcessorLocality.class, JsonProcessorLocalityMixIn.class);
 
     // Register mixins for job coordinator metadata model
-    mapper.getSerializationConfig()
-        .addMixInAnnotations(JobCoordinatorMetadata.class, JsonJobCoordinatorMetadataMixIn.class);
-    mapper.getDeserializationConfig()
-        .addMixInAnnotations(JobCoordinatorMetadata.class, JsonJobCoordinatorMetadataMixIn.class);
+    mapper.addMixIn(JobCoordinatorMetadata.class, JsonJobCoordinatorMetadataMixIn.class);
 
     // Convert camel case to hyphenated field names, and register the module.
     mapper.setPropertyNamingStrategy(new CamelCaseToDashesStrategy());
-    mapper.registerModule(module);
+    mapper.registerModules(module, new Jdk8Module());
 
     return mapper;
   }
@@ -156,7 +160,7 @@ public class SamzaObjectMapper {
     public Config deserialize(JsonParser jsonParser, DeserializationContext context) throws IOException, JsonProcessingException {
       ObjectCodec oc = jsonParser.getCodec();
       JsonNode node = oc.readTree(jsonParser);
-      return new MapConfig(OBJECT_MAPPER.<Map<String, String>>readValue(node, new TypeReference<Map<String, String>>() {
+      return new MapConfig(OBJECT_MAPPER.readValue(OBJECT_MAPPER.treeAsTokens(node), new TypeReference<Map<String, String>>() {
       }));
     }
   }
@@ -164,7 +168,7 @@ public class SamzaObjectMapper {
   public static class PartitionSerializer extends JsonSerializer<Partition> {
     @Override
     public void serialize(Partition partition, JsonGenerator jsonGenerator, SerializerProvider provider) throws IOException, JsonProcessingException {
-      jsonGenerator.writeObject(Integer.valueOf(partition.getPartitionId()));
+      jsonGenerator.writeObject(partition.getPartitionId());
     }
   }
 
@@ -173,7 +177,7 @@ public class SamzaObjectMapper {
     public Partition deserialize(JsonParser jsonParser, DeserializationContext context) throws IOException, JsonProcessingException {
       ObjectCodec oc = jsonParser.getCodec();
       JsonNode node = oc.readTree(jsonParser);
-      return new Partition(node.getIntValue());
+      return new Partition(node.intValue());
     }
   }
 
@@ -189,7 +193,7 @@ public class SamzaObjectMapper {
     public TaskName deserialize(JsonParser jsonParser, DeserializationContext context) throws IOException, JsonProcessingException {
       ObjectCodec oc = jsonParser.getCodec();
       JsonNode node = oc.readTree(jsonParser);
-      return new TaskName(node.getTextValue());
+      return new TaskName(node.textValue());
     }
   }
 
@@ -206,10 +210,10 @@ public class SamzaObjectMapper {
         throws IOException, JsonProcessingException {
       ObjectCodec oc = jsonParser.getCodec();
       JsonNode node = oc.readTree(jsonParser);
-      if (node == null || node.getTextValue().equals("")) {
+      if (node == null || node.textValue().equals("")) {
         return TaskMode.Active;
       } else {
-        return TaskMode.valueOf(node.getTextValue());
+        return TaskMode.valueOf(node.textValue());
       }
     }
   }
@@ -243,6 +247,7 @@ public class SamzaObjectMapper {
       systemStreamPartitionMap.put("system", systemStreamPartition.getSystem());
       systemStreamPartitionMap.put("stream", systemStreamPartition.getStream());
       systemStreamPartitionMap.put("partition", systemStreamPartition.getPartition());
+      systemStreamPartitionMap.put("keyBucket", systemStreamPartition.getKeyBucket());
       jsonGenerator.writeObject(systemStreamPartitionMap);
     }
   }
@@ -252,10 +257,30 @@ public class SamzaObjectMapper {
     public SystemStreamPartition deserialize(JsonParser jsonParser, DeserializationContext context) throws IOException, JsonProcessingException {
       ObjectCodec oc = jsonParser.getCodec();
       JsonNode node = oc.readTree(jsonParser);
-      String system = node.get("system").getTextValue();
-      String stream = node.get("stream").getTextValue();
-      Partition partition = new Partition(node.get("partition").getIntValue());
-      return new SystemStreamPartition(system, stream, partition);
+      String system = node.get("system").textValue();
+      String stream = node.get("stream").textValue();
+      Partition partition = new Partition(node.get("partition").intValue());
+      int keyBucket = -1;
+      if (node.get("keyBucket") != null) {
+        keyBucket = node.get("keyBucket").intValue();
+      }
+      return new SystemStreamPartition(system, stream, partition, keyBucket);
+    }
+  }
+
+  public static class CheckpointIdSerializer extends JsonSerializer<CheckpointId> {
+    @Override
+    public void serialize(CheckpointId checkpointId, JsonGenerator gen, SerializerProvider serializers) throws IOException {
+      gen.writeString(checkpointId.serialize());
+    }
+  }
+
+  public static class CheckpointIdDeserializer extends JsonDeserializer<CheckpointId> {
+    @Override
+    public CheckpointId deserialize(JsonParser jsonParser, DeserializationContext context) throws IOException {
+      ObjectCodec oc = jsonParser.getCodec();
+      JsonNode node = oc.readTree(jsonParser);
+      return CheckpointId.deserialize(node.textValue());
     }
   }
 
