@@ -19,23 +19,18 @@
 
 package org.apache.samza.metrics
 
-import scala.collection._
-import scala.collection.JavaConverters._
-import java.lang.management.ManagementFactory
+import com.google.common.util.concurrent.ThreadFactoryBuilder
+import com.sun.management.OperatingSystemMXBean
+import com.sun.management.UnixOperatingSystemMXBean
+import org.apache.samza.util.Logging
+
 import java.lang.Thread.State._
+import java.lang.management.ManagementFactory
 import java.util.concurrent.Executors
 import java.util.concurrent.TimeUnit
 
-import com.sun.management.{OperatingSystemMXBean, UnixOperatingSystemMXBean}
-import org.apache.samza.util.Logging
-import org.apache.samza.util.DaemonThreadFactory
-
-/**
- *  Companion object for class JvmMetrics encapsulating various constants
- */
-object JvmMetrics {
-  val JVM_METRICS_THREAD_NAME_PREFIX = "JVM-METRICS"
-}
+import scala.collection.JavaConverters._
+import scala.collection._
 
 /**
  * Straight up ripoff of Hadoop's metrics2 JvmMetrics class.
@@ -49,7 +44,8 @@ class JvmMetrics(val registry: MetricsRegistry) extends MetricsHelper with Runna
   val threadMXBean = ManagementFactory.getThreadMXBean()
   val osMXBean = ManagementFactory.getOperatingSystemMXBean()
   var gcBeanCounters = Map[String, (Counter, Counter)]()
-  val executor = Executors.newScheduledThreadPool(1, new DaemonThreadFactory(JvmMetrics.JVM_METRICS_THREAD_NAME_PREFIX))
+  val executor = Executors.newSingleThreadScheduledExecutor(
+    new ThreadFactoryBuilder().setNameFormat("Samza JvmMetrics Thread-%d").setDaemon(true).build())
 
   // jvm metrics
   val gMemNonHeapUsedM = newGauge("mem-non-heap-used-mb", 0.0F)
