@@ -545,17 +545,6 @@ public class TestJobConfig {
     assertEquals(JobConfig.DEFAULT_STANDBY_TASKS_REPLICATION_FACTOR, jobConfig.getStandbyTaskReplicationFactor());
   }
 
-  @Test
-  public void testGetClusterBasedJobCoordinatorDependencyIsolationEnabled() {
-    Config config = new MapConfig(ImmutableMap.of(JobConfig.JOB_SPLIT_DEPLOYMENT_ENABLED, "true"));
-    assertTrue(new JobConfig(config).isSplitDeploymentEnabled());
-
-    config = new MapConfig(ImmutableMap.of(JobConfig.JOB_SPLIT_DEPLOYMENT_ENABLED, "false"));
-    assertFalse(new JobConfig(config).isSplitDeploymentEnabled());
-
-    assertFalse(new JobConfig(new MapConfig()).isSplitDeploymentEnabled());
-  }
-
   @Test
   public void testGetMetadataFile() {
     String execEnvContainerId = "container-id";
@@ -601,4 +590,86 @@ public class TestJobConfig {
     Assert.assertEquals(900, clusterManagerConfig.getContainerMemoryMb());
     Assert.assertEquals(2, clusterManagerConfig.getNumCores());
   }
+
+  @Test
+  public void testGetContainerHeartbeatMonitorEnabled() {
+    assertTrue(new JobConfig(new MapConfig()).getContainerHeartbeatMonitorEnabled());
+    assertTrue(new JobConfig(new MapConfig(
+        ImmutableMap.of(JobConfig.CONTAINER_HEARTBEAT_MONITOR_ENABLED, "true"))).getContainerHeartbeatMonitorEnabled());
+    assertFalse(new JobConfig(new MapConfig(ImmutableMap.of(JobConfig.CONTAINER_HEARTBEAT_MONITOR_ENABLED,
+        "false"))).getContainerHeartbeatMonitorEnabled());
+  }
+
+  @Test
+  public void testGetElastictyEnabled() {
+    // greater than 1 means enabled
+    JobConfig jobConfig = new JobConfig(
+        new MapConfig(ImmutableMap.of(JobConfig.JOB_ELASTICITY_FACTOR, Integer.toString(2))));
+    assertTrue(jobConfig.getElasticityEnabled());
+
+    // one means disabled
+    jobConfig =
+        new JobConfig(new MapConfig(ImmutableMap.of(JobConfig.JOB_ELASTICITY_FACTOR, Integer.toString(1))));
+    assertFalse(jobConfig.getElasticityEnabled());
+
+    // zero means disabled
+    jobConfig =
+        new JobConfig(new MapConfig(ImmutableMap.of(JobConfig.JOB_ELASTICITY_FACTOR, Integer.toString(0))));
+    boolean exceptionCaught = false;
+    try {
+      jobConfig.getElasticityEnabled();
+    } catch (ConfigException e) {
+      exceptionCaught = true;
+    }
+    assertTrue(exceptionCaught);
+
+    // negative means disabled
+    jobConfig =
+        new JobConfig(new MapConfig(ImmutableMap.of(JobConfig.JOB_ELASTICITY_FACTOR, Integer.toString(-1))));
+    exceptionCaught = false;
+    try {
+      jobConfig.getElasticityEnabled();
+    } catch (ConfigException e) {
+      exceptionCaught = true;
+    }
+    assertTrue(exceptionCaught);
+
+    // not specified uses the default standby count, which means disabled
+    jobConfig = new JobConfig(new MapConfig());
+    assertFalse(jobConfig.getElasticityEnabled());
+  }
+
+  @Test
+  public void testGetElasticityFactor() {
+    JobConfig jobConfig = new JobConfig(
+        new MapConfig(ImmutableMap.of(JobConfig.JOB_ELASTICITY_FACTOR, Integer.toString(2))));
+    assertEquals(2, jobConfig.getElasticityFactor());
+
+    jobConfig = new JobConfig(
+        new MapConfig(ImmutableMap.of(JobConfig.JOB_ELASTICITY_FACTOR, Integer.toString(1))));
+    assertEquals(1, jobConfig.getElasticityFactor());
+
+    jobConfig =
+        new JobConfig(new MapConfig(ImmutableMap.of(JobConfig.JOB_ELASTICITY_FACTOR, Integer.toString(0))));
+    boolean exceptionCaught = false;
+    try {
+      jobConfig.getElasticityFactor();
+    } catch (ConfigException e) {
+      exceptionCaught = true;
+    }
+    assertTrue(exceptionCaught);
+
+    jobConfig =
+        new JobConfig(new MapConfig(ImmutableMap.of(JobConfig.JOB_ELASTICITY_FACTOR, Integer.toString(-1))));
+    exceptionCaught = false;
+    try {
+      jobConfig.getElasticityFactor();
+    } catch (ConfigException e) {
+      exceptionCaught = true;
+    }
+    assertTrue(exceptionCaught);
+
+    jobConfig = new JobConfig(new MapConfig());
+    assertEquals(JobConfig.DEFAULT_JOB_ELASTICITY_FACTOR, jobConfig.getElasticityFactor());
+  }
 }
\ No newline at end of file
