@@ -24,6 +24,7 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 import org.apache.samza.application.descriptors.StreamApplicationDescriptorImpl;
+import org.apache.samza.config.MapConfig;
 import org.apache.samza.system.StreamSpec;
 import org.junit.Before;
 import org.junit.Test;
@@ -60,7 +61,7 @@ public class TestJobGraph {
    */
   private void createGraph1() {
     StreamApplicationDescriptorImpl appDesc = mock(StreamApplicationDescriptorImpl.class);
-    graph1 = new JobGraph(null, appDesc);
+    graph1 = new JobGraph(new MapConfig(), appDesc);
 
     JobNode n2 = graph1.getOrCreateJobNode("2", "1");
     JobNode n3 = graph1.getOrCreateJobNode("3", "1");
@@ -94,7 +95,7 @@ public class TestJobGraph {
    */
   private void createGraph2() {
     StreamApplicationDescriptorImpl appDesc = mock(StreamApplicationDescriptorImpl.class);
-    graph2 = new JobGraph(null, appDesc);
+    graph2 = new JobGraph(new MapConfig(), appDesc);
 
     JobNode n1 = graph2.getOrCreateJobNode("1", "1");
     JobNode n2 = graph2.getOrCreateJobNode("2", "1");
@@ -122,7 +123,7 @@ public class TestJobGraph {
    */
   private void createGraph3() {
     StreamApplicationDescriptorImpl appDesc = mock(StreamApplicationDescriptorImpl.class);
-    graph3 = new JobGraph(null, appDesc);
+    graph3 = new JobGraph(new MapConfig(), appDesc);
 
     JobNode n1 = graph3.getOrCreateJobNode("1", "1");
     JobNode n2 = graph3.getOrCreateJobNode("2", "1");
@@ -139,7 +140,7 @@ public class TestJobGraph {
    */
   private void createGraph4() {
     StreamApplicationDescriptorImpl appDesc = mock(StreamApplicationDescriptorImpl.class);
-    graph4 = new JobGraph(null, appDesc);
+    graph4 = new JobGraph(new MapConfig(), appDesc);
 
     JobNode n1 = graph4.getOrCreateJobNode("1", "1");
 
@@ -158,7 +159,7 @@ public class TestJobGraph {
   @Test
   public void testAddSource() {
     StreamApplicationDescriptorImpl appDesc = mock(StreamApplicationDescriptorImpl.class);
-    JobGraph graph = new JobGraph(null, appDesc);
+    JobGraph graph = new JobGraph(new MapConfig(), appDesc);
 
     /**
      * s1 -> 1
@@ -200,7 +201,7 @@ public class TestJobGraph {
      * 2 -> s3
      */
     StreamApplicationDescriptorImpl appDesc = mock(StreamApplicationDescriptorImpl.class);
-    JobGraph graph = new JobGraph(null, appDesc);
+    JobGraph graph = new JobGraph(new MapConfig(), appDesc);
     JobNode n1 = graph.getOrCreateJobNode("1", "1");
     JobNode n2 = graph.getOrCreateJobNode("2", "1");
     StreamSpec s1 = genStream();
