@@ -19,16 +19,20 @@
 package org.apache.samza.operators.impl.store;
 
 import com.google.common.primitives.UnsignedBytes;
+
+import org.apache.samza.checkpoint.CheckpointId;
 import org.apache.samza.serializers.Serde;
 import org.apache.samza.storage.kv.Entry;
 import org.apache.samza.storage.kv.KeyValueSnapshot;
 import org.apache.samza.storage.kv.KeyValueIterator;
 import org.apache.samza.storage.kv.KeyValueStore;
 
+import java.nio.file.Path;
 import java.util.HashMap;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
+import java.util.Optional;
 import java.util.concurrent.ConcurrentNavigableMap;
 import java.util.concurrent.ConcurrentSkipListMap;
 
@@ -129,6 +133,11 @@ public class TestInMemoryStore<K, V> implements KeyValueStore<K, V> {
 
   }
 
+  @Override
+  public Optional<Path> checkpoint(CheckpointId id) {
+    return Optional.empty();
+  }
+
   private static class InMemoryIterator<K, V> implements KeyValueIterator<K, V> {
 
     Iterator<Map.Entry<byte[], byte[]>> wrapped;
