@@ -268,11 +268,10 @@ class TestSamzaContainer extends AssertionsForJUnit with MockitoSugar {
       "0" -> new ContainerModel("0", tasks),
       "1" -> new ContainerModel("1", tasks))
     val jobModel = new JobModel(config, containers)
-    def jobModelGenerator(): Array[Byte] = SamzaObjectMapper.getObjectMapper.writeValueAsBytes(jobModel)
     val server = new HttpServer
     val coordinator = new JobModelManager(jobModel, server)
-    JobModelManager.serializedJobModelRef.set(jobModelGenerator())
-    coordinator.server.addServlet("/*", new JobServlet(JobModelManager.serializedJobModelRef))
+    val serializedJobModel = SamzaObjectMapper.getObjectMapper.writeValueAsBytes(jobModel)
+    coordinator.server.addServlet("/*", new JobServlet(new AtomicReference[Array[Byte]](serializedJobModel)))
     try {
       coordinator.start
       assertEquals(jobModel, SamzaContainer.readJobModel(server.getUrl.toString))
@@ -293,11 +292,10 @@ class TestSamzaContainer extends AssertionsForJUnit with MockitoSugar {
       "0" -> new ContainerModel("0", tasks),
       "1" -> new ContainerModel("1", tasks))
     val jobModel = new JobModel(config, containers)
-    def jobModelGenerator(): Array[Byte] = SamzaObjectMapper.getObjectMapper.writeValueAsBytes(jobModel)
     val server = new HttpServer
     val coordinator = new JobModelManager(jobModel, server)
-    JobModelManager.serializedJobModelRef.set(jobModelGenerator())
-    val mockJobServlet = new MockJobServlet(2, JobModelManager.serializedJobModelRef)
+    val serializedJobModel = SamzaObjectMapper.getObjectMapper.writeValueAsBytes(jobModel)
+    val mockJobServlet = new MockJobServlet(2, new AtomicReference[Array[Byte]](serializedJobModel))
     coordinator.server.addServlet("/*", mockJobServlet)
     try {
       coordinator.start
@@ -308,40 +306,6 @@ class TestSamzaContainer extends AssertionsForJUnit with MockitoSugar {
     assertEquals(2, mockJobServlet.exceptionCount)
   }
 
-  @Test
-  def testGetChangelogSSPsForContainer() {
-    val taskName0 = new TaskName("task0")
-    val taskName1 = new TaskName("task1")
-    val taskModel0 = new TaskModel(taskName0,
-      Set(new SystemStreamPartition("input", "stream", new Partition(0))),
-      new Partition(10))
-    val taskModel1 = new TaskModel(taskName1,
-      Set(new SystemStreamPartition("input", "stream", new Partition(1))),
-      new Partition(11))
-    val containerModel = new ContainerModel("processorId", Map(taskName0 -> taskModel0, taskName1 -> taskModel1))
-    val changeLogSystemStreams = Map("store0" -> new SystemStream("changelogSystem0", "store0-changelog"),
-      "store1" -> new SystemStream("changelogSystem1", "store1-changelog"))
-    val expected = Set(new SystemStreamPartition("changelogSystem0", "store0-changelog", new Partition(10)),
-      new SystemStreamPartition("changelogSystem1", "store1-changelog", new Partition(10)),
-      new SystemStreamPartition("changelogSystem0", "store0-changelog", new Partition(11)),
-      new SystemStreamPartition("changelogSystem1", "store1-changelog", new Partition(11)))
-    assertEquals(expected, SamzaContainer.getChangelogSSPsForContainer(containerModel, changeLogSystemStreams))
-  }
-
-  @Test
-  def testGetChangelogSSPsForContainerNoChangelogs() {
-    val taskName0 = new TaskName("task0")
-    val taskName1 = new TaskName("task1")
-    val taskModel0 = new TaskModel(taskName0,
-      Set(new SystemStreamPartition("input", "stream", new Partition(0))),
-      new Partition(10))
-    val taskModel1 = new TaskModel(taskName1,
-      Set(new SystemStreamPartition("input", "stream", new Partition(1))),
-      new Partition(11))
-    val containerModel = new ContainerModel("processorId", Map(taskName0 -> taskModel0, taskName1 -> taskModel1))
-    assertEquals(Set(), SamzaContainer.getChangelogSSPsForContainer(containerModel, Map()))
-  }
-
   @Test
   def testStoreContainerLocality():Unit = {
     this.config = new MapConfig(Map(ClusterManagerConfig.JOB_HOST_AFFINITY_ENABLED -> "true"))
