@@ -22,20 +22,19 @@ package org.apache.samza.checkpoint.kafka
 import java.util.Collections
 import java.util.concurrent.TimeUnit
 import java.util.concurrent.atomic.AtomicReference
-
 import com.google.common.annotations.VisibleForTesting
 import com.google.common.base.Preconditions
-import org.apache.samza.checkpoint.{Checkpoint, CheckpointManager}
+import org.apache.samza.checkpoint.{Checkpoint, CheckpointManager, CheckpointV1, CheckpointV2}
 import org.apache.samza.config.{Config, JobConfig, TaskConfig}
 import org.apache.samza.container.TaskName
-import org.apache.samza.serializers.Serde
+import org.apache.samza.serializers.{CheckpointV1Serde, CheckpointV2Serde, Serde}
 import org.apache.samza.metrics.MetricsRegistry
-import org.apache.samza.serializers.CheckpointSerde
 import org.apache.samza.system._
 import org.apache.samza.system.kafka.KafkaStreamSpec
-import org.apache.samza.util.{ExponentialSleepStrategy, Logging}
+import org.apache.samza.util.Logging
 import org.apache.samza.{Partition, SamzaException}
 
+import java.{lang, util}
 import scala.collection.mutable
 
 /**
@@ -53,7 +52,8 @@ class KafkaCheckpointManager(checkpointSpec: KafkaStreamSpec,
                              validateCheckpoint: Boolean,
                              config: Config,
                              metricsRegistry: MetricsRegistry,
-                             checkpointMsgSerde: Serde[Checkpoint] = new CheckpointSerde,
+                             checkpointV1MsgSerde: Serde[CheckpointV1] = new CheckpointV1Serde,
+                             checkpointV2MsgSerde: Serde[CheckpointV2] = new CheckpointV2Serde,
                              checkpointKeySerde: Serde[KafkaCheckpointLogKey] = new KafkaCheckpointLogKeySerde) extends CheckpointManager with Logging {
 
   var MaxRetryDurationInMillis: Long = TimeUnit.MINUTES.toMillis(15)
@@ -81,6 +81,10 @@ class KafkaCheckpointManager(checkpointSpec: KafkaStreamSpec,
   // for active containers, this will be set to true, while false for standby containers.
   val stopConsumerAfterFirstRead: Boolean = new TaskConfig(config).getCheckpointManagerConsumerStopAfterFirstRead
 
+  val checkpointReadVersions: util.List[lang.Short] = new TaskConfig(config).getCheckpointReadVersions
+  val liveCheckpointMaxAgeMillis: Long = new TaskConfig(config).getLiveCheckpointMaxAgeMillis
+
+
   /**
     * Create checkpoint stream prior to start.
     *
@@ -159,19 +163,7 @@ class KafkaCheckpointManager(checkpointSpec: KafkaStreamSpec,
     * @inheritdoc
     */
   override def writeCheckpoint(taskName: TaskName, checkpoint: Checkpoint) {
-    val key = new KafkaCheckpointLogKey(KafkaCheckpointLogKey.CHECKPOINT_KEY_TYPE, taskName, expectedGrouperFactory)
-    val keyBytes = try {
-      checkpointKeySerde.toBytes(key)
-    } catch {
-      case e: Exception => throw new SamzaException(s"Exception when writing checkpoint-key for $taskName: $checkpoint", e)
-    }
-    val msgBytes = try {
-      checkpointMsgSerde.toBytes(checkpoint)
-    } catch {
-      case e: Exception => throw new SamzaException(s"Exception when writing checkpoint for $taskName: $checkpoint", e)
-    }
-
-    val envelope = new OutgoingMessageEnvelope(checkpointSsp, keyBytes, msgBytes)
+    val envelope = buildOutgoingMessageEnvelope(taskName, checkpoint)
 
     // Used for exponential backoff retries on failure in sending messages through producer.
     val startTimeInMillis: Long = System.currentTimeMillis()
@@ -188,7 +180,7 @@ class KafkaCheckpointManager(checkpointSpec: KafkaStreamSpec,
       } catch {
         case exception: Exception => {
           producerException = exception
-          warn(s"Retrying failed checkpoint write to key: $key, checkpoint: $checkpoint for task: $taskName", exception)
+          warn(s"Retrying failed write for checkpoint: $checkpoint for task: $taskName", exception)
           // TODO: Remove this producer recreation logic after SAMZA-1393.
           val newProducer: SystemProducer = getSystemProducer()
           producerCreationLock.synchronized {
@@ -253,12 +245,15 @@ class KafkaCheckpointManager(checkpointSpec: KafkaStreamSpec,
     */
   private def readCheckpoints(): Map[TaskName, Checkpoint] = {
     val checkpoints = mutable.Map[TaskName, Checkpoint]()
+    val checkpointAppendTime = mutable.Map[TaskName, Long]()
 
     val iterator = new SystemStreamPartitionIterator(systemConsumer, checkpointSsp)
     var numMessagesRead = 0
 
     while (iterator.hasNext) {
       val checkpointEnvelope: IncomingMessageEnvelope = iterator.next
+      // Kafka log append time for the checkpoint message
+      val checkpointEnvelopeTs = checkpointEnvelope.getEventTime;
       val offset = checkpointEnvelope.getOffset
 
       numMessagesRead += 1
@@ -276,10 +271,10 @@ class KafkaCheckpointManager(checkpointSpec: KafkaStreamSpec,
         checkpointKeySerde.fromBytes(keyBytes)
       } catch {
         case e: Exception => if (validateCheckpoint) {
-          throw new SamzaException(s"Exception while serializing checkpoint-key. " +
+          throw new SamzaException(s"Exception while deserializing checkpoint-key. " +
             s"Topic: $checkpointTopic Offset: $offset", e)
         } else {
-          warn(s"Ignoring exception while serializing checkpoint-key. Topic: $checkpointTopic Offset: $offset", e)
+          warn(s"Ignoring exception while deserializing checkpoint-key. Topic: $checkpointTopic Offset: $offset", e)
           null
         }
       }
@@ -295,17 +290,31 @@ class KafkaCheckpointManager(checkpointSpec: KafkaStreamSpec,
           }
         }
 
-        // If the type of the key is not KafkaCheckpointLogKey.CHECKPOINT_KEY_TYPE, it can safely be ignored.
-        if (KafkaCheckpointLogKey.CHECKPOINT_KEY_TYPE.equals(checkpointKey.getType)) {
-          val checkpointBytes = checkpointEnvelope.getMessage.asInstanceOf[Array[Byte]]
-          val checkpoint = try {
-            checkpointMsgSerde.fromBytes(checkpointBytes)
-          } catch {
-            case e: Exception => throw new SamzaException(s"Exception while serializing checkpoint-message. " +
-              s"Topic: $checkpointTopic Offset: $offset", e)
+        val msgBytes = checkpointEnvelope.getMessage.asInstanceOf[Array[Byte]]
+        try {
+          // if checkpoint key version does not match configured checkpoint version to read, skip the message.
+          if (checkpointReadVersions.contains(
+            KafkaCheckpointLogKey.CHECKPOINT_KEY_VERSIONS.get(checkpointKey.getType))) {
+            val taskName = checkpointKey.getTaskName
+            if (!checkpoints.contains(taskName) ||
+              shouldOverrideCheckpoint(checkpoints.get(taskName), checkpointKey, checkpointAppendTime.get(taskName),
+                checkpointEnvelopeTs)) {
+              checkpoints.put(taskName, deserializeCheckpoint(checkpointKey, msgBytes))
+              checkpointAppendTime.put(taskName, checkpointEnvelopeTs)
+            } // else ignore the de-prioritized checkpoint
+          } else {
+            // Ignore and skip the unknown checkpoint key type. We do not want to throw any exceptions for this case
+            // for forwards compatibility with new checkpoints versions in the checkpoint topic
+            warn(s"Ignoring unknown checkpoint key type for checkpoint key: $checkpointKey")
           }
-
-          checkpoints.put(checkpointKey.getTaskName, checkpoint)
+        } catch {
+          case e: Exception =>
+            if (validateCheckpoint) {
+              throw new SamzaException(s"Exception while deserializing checkpoint-message. " +
+                s"Topic: $checkpointTopic Offset: $offset", e)
+            } else {
+              warn(s"Ignoring exception while deserializing checkpoint-msg. Topic: $checkpointTopic Offset: $offset", e)
+            }
         }
       }
     }
@@ -333,4 +342,72 @@ class KafkaCheckpointManager(checkpointSpec: KafkaStreamSpec,
 
     partitionMetaData.getOldestOffset
   }
+
+  def buildOutgoingMessageEnvelope[T <: Checkpoint](taskName: TaskName, checkpoint: T): OutgoingMessageEnvelope = {
+    checkpoint match {
+      case checkpointV1: CheckpointV1 => {
+        val key = new KafkaCheckpointLogKey(
+          KafkaCheckpointLogKey.CHECKPOINT_V1_KEY_TYPE, taskName, expectedGrouperFactory)
+        val keyBytes = try {
+          checkpointKeySerde.toBytes(key)
+        } catch {
+          case e: Exception =>
+            throw new SamzaException(s"Exception when writing checkpoint-key for $taskName: $checkpoint", e)
+        }
+        val msgBytes = try {
+          checkpointV1MsgSerde.toBytes(checkpointV1)
+        } catch {
+          case e: Exception =>
+            throw new SamzaException(s"Exception when writing checkpoint for $taskName: $checkpoint", e)
+        }
+        new OutgoingMessageEnvelope(checkpointSsp, keyBytes, msgBytes)
+      }
+      case checkpointV2: CheckpointV2 => {
+        val key = new KafkaCheckpointLogKey(
+          KafkaCheckpointLogKey.CHECKPOINT_V2_KEY_TYPE, taskName, expectedGrouperFactory)
+        val keyBytes = try {
+          checkpointKeySerde.toBytes(key)
+        } catch {
+          case e: Exception =>
+            throw new SamzaException(s"Exception when writing checkpoint-key for $taskName: $checkpoint", e)
+        }
+        val msgBytes = try {
+          checkpointV2MsgSerde.toBytes(checkpointV2)
+        } catch {
+          case e: Exception =>
+            throw new SamzaException(s"Exception when writing checkpoint for $taskName: $checkpoint", e)
+        }
+        new OutgoingMessageEnvelope(checkpointSsp, keyBytes, msgBytes)
+      }
+      case _ => throw new SamzaException("Unknown checkpoint version: " + checkpoint.getVersion)
+    }
+  }
+
+  private def shouldOverrideCheckpoint(currentCheckpoint: Option[Checkpoint], newCheckpointKey: KafkaCheckpointLogKey,
+    currentCheckpointAppendTime: Option[Long], newCheckpointAppendTime: Long): Boolean = {
+    val newCheckpointVersion = KafkaCheckpointLogKey.CHECKPOINT_KEY_VERSIONS.get(newCheckpointKey.getType)
+    if (newCheckpointVersion == null) {
+      // Unknown checkpoint version
+      throw new IllegalArgumentException("Unknown checkpoint key type: " + newCheckpointKey.getType +
+        " for checkpoint key: " + newCheckpointKey)
+    }
+    // Override checkpoint if:
+    // 1. The current checkpoint does not exist or
+    // 2. The new checkpoint has a higher restore priority than the currently written checkpoint
+    // 3. The current checkpoint is determined to be stale compared to the new checkpoint timestamp
+    currentCheckpoint.isEmpty ||
+      checkpointReadVersions.indexOf(newCheckpointVersion) <=
+        checkpointReadVersions.indexOf(currentCheckpoint.get.getVersion) ||
+      (newCheckpointAppendTime - currentCheckpointAppendTime.get > liveCheckpointMaxAgeMillis)
+  }
+
+  private def deserializeCheckpoint(checkpointKey: KafkaCheckpointLogKey, checkpointMsgBytes: Array[Byte]): Checkpoint = {
+    if (KafkaCheckpointLogKey.CHECKPOINT_V1_KEY_TYPE.equals(checkpointKey.getType)) {
+      checkpointV1MsgSerde.fromBytes(checkpointMsgBytes)
+    } else if (KafkaCheckpointLogKey.CHECKPOINT_V2_KEY_TYPE.equals(checkpointKey.getType)) {
+      checkpointV2MsgSerde.fromBytes(checkpointMsgBytes)
+    } else {
+      throw new IllegalArgumentException("Unknown checkpoint key type: " + checkpointKey.getType)
+    }
+  }
 }
