@@ -22,11 +22,10 @@ package org.apache.samza.storage.kv
 
 import java.io.File
 import java.util
-
 import org.apache.samza.SamzaException
 import org.apache.samza.config.MapConfig
 import org.apache.samza.metrics.{Counter, Gauge, MetricsRegistryMap, MetricsVisitor, Timer}
-import org.apache.samza.util.ExponentialSleepStrategy
+import org.apache.samza.util.{ExponentialSleepStrategy, FileUtil}
 import org.junit.{Assert, Test}
 import org.rocksdb.{FlushOptions, Options, RocksDB, RocksIterator}
 
@@ -123,6 +122,23 @@ class TestRocksDbKeyValueStore
     rocksDB.get(key)
   }
 
+  @Test
+  def testRocksDbCreatePathIfNotExist(): Unit = {
+    val map = new util.HashMap[String, String]()
+    val config = new MapConfig(map)
+    val options = new Options()
+    options.setCreateIfMissing(true)
+
+    val dbDir = new File(System.getProperty("java.io.tmpdir") + File.separator + "samza-test2", "rocksDbFiles")
+    val rocksDB = new RocksDbKeyValueStore(dbDir, options, config, false, "dbStore")
+    val key = "key".getBytes("UTF-8")
+    rocksDB.put(key, "val".getBytes("UTF-8"))
+
+    rocksDB.close()
+
+    new FileUtil().rm(dbDir)
+  }
+
   @Test
   def testIteratorWithRemoval(): Unit = {
     val lock = new Object
@@ -183,7 +199,7 @@ class TestRocksDbKeyValueStore
 
   @Test
   def testMetricsConfig(): Unit = {
-    val registry = new MetricsRegistryMap("registrymap")
+    val registry = new MetricsRegistryMap()
     val metrics = new KeyValueStoreMetrics("dbstore", registry)
 
     val map = new util.HashMap[String, String]()
@@ -213,7 +229,7 @@ class TestRocksDbKeyValueStore
 
   @Test
   def testRocksDBMetricsWithBulkLoadRWRecreate(): Unit = {
-    val registry = new MetricsRegistryMap("registrymap")
+    val registry = new MetricsRegistryMap()
     val metrics = new KeyValueStoreMetrics("dbstore", registry)
 
     // Sample metric values for estimate-num-keys metrics
