@@ -19,7 +19,7 @@
 package org.apache.samza.rest.model;
 
 import java.util.List;
-import org.codehaus.jackson.annotate.JsonProperty;
+import com.fasterxml.jackson.annotation.JsonProperty;
 
 
 /**
