@@ -28,12 +28,11 @@ import javax.ws.rs.core.MediaType;
 import javax.ws.rs.core.Response;
 import org.apache.commons.lang.StringUtils;
 import org.apache.samza.SamzaException;
-import org.apache.samza.rest.model.Task;
 import org.apache.samza.rest.proxy.job.JobInstance;
 import org.apache.samza.rest.proxy.task.TaskProxyFactory;
 import org.apache.samza.rest.proxy.task.TaskProxy;
 import org.apache.samza.rest.proxy.task.TaskResourceConfig;
-import org.apache.samza.util.Util;
+import org.apache.samza.util.ReflectionUtil;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -59,7 +58,7 @@ public class TasksResource {
     Preconditions.checkArgument(StringUtils.isNotEmpty(taskProxyFactoryClassName),
                                 String.format("Missing config: %s", TaskResourceConfig.CONFIG_TASK_PROXY_FACTORY));
     try {
-      TaskProxyFactory factory = Util.getObj(taskProxyFactoryClassName, TaskProxyFactory.class);
+      TaskProxyFactory factory = ReflectionUtil.getObj(taskProxyFactoryClassName, TaskProxyFactory.class);
       taskProxy = factory.getTaskProxy(config);
     } catch (Exception e) {
       LOG.error(String.format("Exception in building TasksResource with config: %s.", config), e);
@@ -68,11 +67,11 @@ public class TasksResource {
   }
 
   /**
-   * Gets the list of {@link Task} for the job instance specified by jobName and jobId.
+   * Gets the list of {@link org.apache.samza.rest.model.Task} for the job instance specified by jobName and jobId.
    * @param jobName the name of the job as configured in {@link org.apache.samza.config.JobConfig#JOB_NAME}
    * @param jobId the id of the job as configured in {@link org.apache.samza.config.JobConfig#JOB_ID}.
    * @return a {@link javax.ws.rs.core.Response.Status#OK} {@link javax.ws.rs.core.Response}
-   *         contains a list of {@link Task}, where each task belongs to
+   *         contains a list of {@link org.apache.samza.rest.model.Task}, where each task belongs to
    *         the samza job. {@link javax.ws.rs.core.Response.Status#BAD_REQUEST} is returned for invalid
    *         job instances.
    */
