@@ -19,11 +19,12 @@
 
 package org.apache.samza.sql.client.cli;
 
+import org.apache.samza.sql.client.exceptions.ExecutorException;
 import org.apache.samza.sql.client.interfaces.QueryResult;
 
 /**
  * For displaying the streaming result of a SELECT statement.
  */
 public interface CliView {
-  public void open(CliShell shell, QueryResult queryResult);
+  public void open(CliShell shell, QueryResult queryResult) throws ExecutorException;
 }
