@@ -21,17 +21,20 @@ package org.apache.samza.sql.runner;
 
 import java.util.Collection;
 import java.util.HashMap;
-import java.util.HashSet;
+import java.util.LinkedList;
 import java.util.List;
 import java.util.Map;
-import java.util.Set;
-import java.util.concurrent.atomic.AtomicInteger;
 import org.apache.calcite.rel.RelRoot;
 import org.apache.samza.application.StreamApplication;
 import org.apache.samza.application.descriptors.StreamApplicationDescriptor;
+import org.apache.samza.context.ApplicationContainerContext;
+import org.apache.samza.context.ApplicationTaskContextFactory;
+import org.apache.samza.context.ContainerContext;
+import org.apache.samza.context.ExternalContext;
+import org.apache.samza.context.JobContext;
+import org.apache.samza.context.TaskContext;
 import org.apache.samza.sql.data.SamzaSqlExecutionContext;
 import org.apache.samza.sql.dsl.SamzaSqlDslConverter;
-import org.apache.samza.sql.interfaces.SamzaRelConverter;
 import org.apache.samza.sql.translator.QueryTranslator;
 import org.apache.samza.sql.translator.TranslatorContext;
 import org.slf4j.Logger;
@@ -44,7 +47,6 @@ import org.slf4j.LoggerFactory;
 public class SamzaSqlApplication implements StreamApplication {
 
   private static final Logger LOG = LoggerFactory.getLogger(SamzaSqlApplication.class);
-  private AtomicInteger queryId = new AtomicInteger(0);
 
   @Override
   public void describe(StreamApplicationDescriptor appDescriptor) {
@@ -55,8 +57,8 @@ public class SamzaSqlApplication implements StreamApplication {
       Map<Integer, TranslatorContext> translatorContextMap = new HashMap<>();
 
       // 1. Get Calcite plan
-      Set<String> inputSystemStreams = new HashSet<>();
-      Set<String> outputSystemStreams = new HashSet<>();
+      List<String> inputSystemStreams = new LinkedList<>();
+      List<String> outputSystemStreams = new LinkedList<>();
 
       Collection<RelRoot> relRoots =
           SamzaSqlApplicationConfig.populateSystemStreamsAndGetRelRoots(dslStmts, appDescriptor.getConfig(),
@@ -69,13 +71,14 @@ public class SamzaSqlApplication implements StreamApplication {
       // 3. Translate Calcite plan to Samza stream operators
       QueryTranslator queryTranslator = new QueryTranslator(appDescriptor, sqlConfig);
       SamzaSqlExecutionContext executionContext = new SamzaSqlExecutionContext(sqlConfig);
-      Map<String, SamzaRelConverter> converters = sqlConfig.getSamzaRelConverters();
+      // QueryId implies the index of the query in multiple query statements scenario. It should always start with 0.
+      int queryId = 0;
       for (RelRoot relRoot : relRoots) {
-        LOG.info("Translating relRoot {} to samza stream graph", relRoot);
-        int qId = queryId.incrementAndGet();
-        TranslatorContext translatorContext = new TranslatorContext(appDescriptor, relRoot, executionContext, converters);
-        translatorContextMap.put(qId, translatorContext);
-        queryTranslator.translate(relRoot, translatorContext, qId);
+        LOG.info("Translating relRoot {} to samza stream graph with queryId {}", relRoot, queryId);
+        TranslatorContext translatorContext = new TranslatorContext(appDescriptor, relRoot, executionContext);
+        translatorContextMap.put(queryId, translatorContext);
+        queryTranslator.translate(relRoot, sqlConfig.getOutputSystemStreams().get(queryId), translatorContext, queryId);
+        queryId++;
       }
 
       // 4. Set all translator contexts
@@ -85,11 +88,14 @@ public class SamzaSqlApplication implements StreamApplication {
        * container, so it does not need to be serialized. Therefore, the translatorContext is recreated in each container
        * and does not need to be serialized.
        */
-      appDescriptor.withApplicationTaskContextFactory((jobContext,
-          containerContext,
-          taskContext,
-          applicationContainerContext) ->
-          new SamzaSqlApplicationContext(translatorContextMap));
+      appDescriptor.withApplicationTaskContextFactory(new ApplicationTaskContextFactory<SamzaSqlApplicationContext>() {
+        @Override
+        public SamzaSqlApplicationContext create(ExternalContext externalContext, JobContext jobContext,
+            ContainerContext containerContext, TaskContext taskContext,
+            ApplicationContainerContext applicationContainerContext) {
+          return new SamzaSqlApplicationContext(translatorContextMap);
+        }
+      });
     } catch (RuntimeException e) {
       LOG.error("SamzaSqlApplication threw exception.", e);
       throw e;
